package com.crypto.utils;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.SecureRandom;
import java.security.Security;
import java.security.SignatureException;
import java.security.cert.CRLException;
import java.security.cert.Certificate;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.security.spec.ECGenParameterSpec;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.naming.NamingException;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.keyinfo.KeyInfoFactory;
import javax.xml.crypto.dsig.keyinfo.X509IssuerSerial;
import javax.xml.parsers.ParserConfigurationException;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.UUID;

import javax.xml.crypto.XMLStructure;
import javax.xml.crypto.dom.DOMStructure;
import javax.xml.crypto.dsig.CanonicalizationMethod;
import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.SignedInfo;
import javax.xml.crypto.dsig.XMLObject;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.X509Data;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.dom.DOMSource;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.crypto.AlgorithmMethod;
import javax.xml.crypto.KeySelector;
import javax.xml.crypto.KeySelectorException;
import javax.xml.crypto.KeySelectorResult;
import javax.xml.crypto.XMLCryptoContext;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.cms.CMSAlgorithm;
import org.bouncycastle.cms.CMSEnvelopedData;
import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.CMSProcessableByteArray;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.cms.CMSSignedDataGenerator;
import org.bouncycastle.cms.CMSTypedData;
import org.bouncycastle.cms.RecipientInformation;
import org.bouncycastle.cms.RecipientInformationStore;
import org.bouncycastle.cms.Recipient;
import org.bouncycastle.cms.SignerId;
import org.bouncycastle.cms.SignerInformation;
import org.bouncycastle.cms.SignerInformationStore;
import org.bouncycastle.cms.jcajce.JcaSignerInfoGeneratorBuilder;
import org.bouncycastle.cms.jcajce.JcaSimpleSignerInfoVerifierBuilder;
import org.bouncycastle.cms.jcajce.JceCMSContentEncryptorBuilder;
import org.bouncycastle.cms.jcajce.JceKeyTransEnvelopedRecipient;
import org.bouncycastle.cms.jcajce.JceKeyTransRecipientInfoGenerator;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.operator.ContentSigner;
import org.bouncycastle.operator.DefaultSignatureAlgorithmIdentifierFinder;
import org.bouncycastle.operator.OperatorCreationException;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
import org.bouncycastle.operator.jcajce.JcaDigestCalculatorProviderBuilder;

import com.crypto.utils.helpers.AlgorithmIDf;
import com.crypto.utils.helpers.Base64Utils;
import com.crypto.utils.helpers.CRLHelper;

import com.crypto.utils.helpers.CerHelper;
import com.crypto.utils.helpers.ConfigClass;
import com.crypto.utils.helpers.KeyStoreHolder;
import com.crypto.utils.helpers.KeyStoreLoader;
import com.crypto.utils.helpers.LdapReader;
import com.crypto.utils.helpers.XmlReader;
import com.sun.crypto.provider.SunJCE;
import org.dom4j.tree.AbstractText;

public class AzTrustCryptoLib {
	public static final String WINDOWS_CONTAINER_NAME = "WINDOWS";
	public static final String PKCS11_CONTAINER_NAME = "PKCS11";
	public static final String PKCS12_CONTAINER_NAME = "PFX_FILE";
	public static final String JKS_CONTAINER_NAME = "JKS_FILE";
	public static String CONFIG_XML_FILE_PATH = "";

	static {
		URL location = AzTrustCryptoLib.class.getProtectionDomain().getCodeSource().getLocation();

		String os = System.getProperty("os.name").toLowerCase();
		if (os.contains("nix") || os.contains("aix") || os.contains("nux")) {
			CONFIG_XML_FILE_PATH = "/" + getXmlPath(location.getPath()) + "AzTrustCrypto.xml";
		} else {
			CONFIG_XML_FILE_PATH = getXmlPath(location.getPath()) + "AzTrustCrypto.xml";
		}
	}

	// ****************************************************************
	// PUBLIC METHODS BEGIN *
	// ****************************************************************

	public String signData(byte[] inputdata, String cer_issuer, String cer_serial, String hash_oid,
			String containerName, String containerPassword, String containerFilePath, int detached)
			throws KeyStoreException, GeneralSecurityException, IOException, SecurityException,
			IllegalArgumentException, ClassNotFoundException, NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException, OperatorCreationException, CMSException {
		KeyStoreHolder storeHolder = getKeyStoreHolder(cer_issuer, cer_serial, containerName, containerPassword,
				containerFilePath);
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null && !containerPassword.isEmpty() && !containerPassword.equals(""))
				passw = containerPassword.toCharArray();
			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(storeHolder.getAlias(), passw);
			return Base64Utils.base64Encode(sign(inputdata, hash_oid, pk, storeHolder, detached));
		}
		return null;
	}

	public String signData(String inputdata, String cer_issuer, String cer_serial, String hash_oid,
			String containerName, String containerPassword, String containerFilePath, int detached)
			throws KeyStoreException, GeneralSecurityException, IOException, SecurityException,
			IllegalArgumentException, ClassNotFoundException, NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException, OperatorCreationException, CMSException {
		byte[] inputDataBytes = inputdata.getBytes();
		return signData(inputDataBytes, cer_issuer, cer_serial, hash_oid, containerName, containerPassword,
				containerFilePath, detached);
	}

	public String signData(String inputdata, String cer_id) throws KeyStoreException, GeneralSecurityException,
			IOException, SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException, InvocationTargetException, OperatorCreationException,
			CMSException, ParserConfigurationException, SAXException {
		byte[] inputDataBytes = inputdata.getBytes();
		return signData(inputDataBytes, cer_id);
	}

	public String signData(byte[] inputdata, String cer_id) throws KeyStoreException, GeneralSecurityException,
			IOException, SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException, InvocationTargetException, OperatorCreationException,
			CMSException, ParserConfigurationException, SAXException {
		ConfigClass conf = LoadConfig(cer_id);
		return signData(inputdata, conf.CerIssuer, conf.CerSerial, conf.HashOid, conf.ContainerName,
				conf.ContainerPassword, conf.ContainerFilePath, conf.Detached);
	}

	public String verifyData(String inputdata, String CMSSignature, String ldapServer, String ldapPort,
			String ldapCertSearchDN, String ldp_usr, String ldp_pass) throws CMSException, OperatorCreationException,
			CertificateException, NamingException, ParserConfigurationException, SAXException, IOException {
		byte[] inputDataBytes = inputdata.getBytes();
		return verifyData(inputDataBytes, CMSSignature, ldapServer, ldapPort, ldapCertSearchDN, ldp_usr, ldp_pass);
	}

	public String verifyData(byte[] inputdata, String CMSSignature, String ldapServer, String ldapPort,
			String ldapCertSearchDN, String ldp_usr, String ldp_pass) throws CMSException, OperatorCreationException,
			CertificateException, NamingException, ParserConfigurationException, SAXException, IOException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(CMSSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputdata);
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();

		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();
		String ldapUser = ldp_usr;
		String ldapPassword = ldp_pass;

		String Issuer = CerHelper.toStringReversed(X500Name.getInstance(signerId.getIssuer().getEncoded()));
		Issuer = CorrectIssuerDN(Issuer);
		String hexString = signerId.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();
		while ((sb.length() + hexString.length()) < (2 * signerId.getSerialNumber().toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		String serialNumber = sb.toString();

		X509Certificate cert = getCertificateX509(ldapServer, ldapPort, ldapCertSearchDN, ldapUser, ldapPassword,
				Issuer, serialNumber);
		boolean verified = signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert));
		if (verified)
			return "verified";
		else
			return "not verified";
	}

	public String verifyData(byte[] inputdata, String CMSSignature, String signercer)
			throws CMSException, OperatorCreationException, CertificateException, KeyStoreException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(CMSSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputdata);
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		X509Certificate cert = getCertFromString(signercer);
		Security.addProvider(new BouncyCastleProvider());
		boolean verified = signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert));
		if (verified)
			return "verified";
		else
			return "not verified";
	}

	public String verifyData(String inputdata, String CMSSignature, String signercer)
			throws CMSException, OperatorCreationException, CertificateException, KeyStoreException {
		byte[] inputDataBytes = inputdata.getBytes();
		return verifyData(inputDataBytes, CMSSignature, signercer);
	}

	public String verifyData(byte[] inputdata, String CMSSignature) throws CMSException, OperatorCreationException,
			CertificateException, NamingException, ParserConfigurationException, SAXException, IOException {
		ConfigClass conf = LoadConfig("");
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(CMSSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputdata);
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();
		String Issuer = CerHelper.toStringReversed(X500Name.getInstance(signerId.getIssuer().getEncoded()));
		Issuer = CorrectIssuerDN(Issuer);
		String hexString = signerId.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();
		while ((sb.length() + hexString.length()) < (2 * signerId.getSerialNumber().toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		String serialNumber = sb.toString();
		X509Certificate cert = getCertificateX509(conf.LdapServer, conf.LdapPort, conf.LdapCertSearchDN, conf.LdapUser,
				conf.LdapPassword, Issuer, serialNumber);
		Security.addProvider(new BouncyCastleProvider());
		boolean verified = signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert));
		if (verified)
			return "verified";
		else
			return "not verified";
	}

	public String verifyData(String inputdata, String CMSSignature) throws CMSException, OperatorCreationException,
			CertificateException, NamingException, ParserConfigurationException, SAXException, IOException {
		byte[] inputDataBytes = inputdata.getBytes();
		return verifyData(inputDataBytes, CMSSignature);
	}

	public byte[] normalize(org.dom4j.Element elem) throws UnsupportedEncodingException {
		final StringBuilder strb = new StringBuilder();
		canonicalize(elem, strb);
		return strb.toString().getBytes("UTF-16LE");
	}

	public String encryptData(String inputdata, String recipientCert)
			throws IOException, CertificateException, KeyStoreException {
		byte[] inputd = inputdata.getBytes();
		X509Certificate recipientCer = getCertFromString(recipientCert);
		CMSEnvelopedDataGenerator edGen = new CMSEnvelopedDataGenerator();
		CMSEnvelopedData ed;
		Security.addProvider(new BouncyCastleProvider());
		try {
			Security.insertProviderAt(new SunJCE(), 1);
			edGen.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(recipientCer).setProvider("BC"));
			ed = edGen.generate(new CMSProcessableByteArray(inputd),
					new JceCMSContentEncryptorBuilder(CMSAlgorithm.DES_EDE3_CBC).setProvider("BC").build());
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}
		return Base64Utils.base64Encode(ed.getEncoded());
	}

	public String decrypteData(String inputdata, String cer_issuer, String cer_serial, String hash_oid,
			String containerName, String containerPassword, String containerFilePath)
			throws KeyStoreException, GeneralSecurityException, IOException, SecurityException,
			IllegalArgumentException, ClassNotFoundException, NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException, OperatorCreationException, CMSException {
		KeyStoreHolder storeHolder = getKeyStoreHolder(cer_issuer, cer_serial, containerName, containerPassword,
				containerFilePath);
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null && !containerPassword.isEmpty() && !containerPassword.equals(""))
				passw = containerPassword.toCharArray();
			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(storeHolder.getAlias(), passw);
			return new String(decrypt(inputdata, pk, storeHolder), "UTF-8");
		}
		return null;
	}

	public String getCMSSerNum(String cmsSignature)
			throws CMSException, NoSuchAlgorithmException, NoSuchProviderException {
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSSignedData signedData = new CMSSignedData(signature);
		SignerInformationStore signers = signedData.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		return getSerial(signer.getSID().getSerialNumber());
	}

	public String getCMSAlgorithmName(String cmsSignature)
			throws CMSException, NoSuchAlgorithmException, NoSuchProviderException {
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSSignedData signedData = new CMSSignedData(signature);
		SignerInformationStore signers = signedData.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		return AlgorithmIDf.getSignatureAlgorithmName(signer.toASN1Structure().getDigestAlgorithm(),
				signer.toASN1Structure().getDigestEncryptionAlgorithm());
	}

	public String getCMSAlgorithmOID(String cmsSignature)
			throws CMSException, NoSuchAlgorithmException, NoSuchProviderException {
		AlgorithmIdentifier sigAlg = new DefaultSignatureAlgorithmIdentifierFinder()
				.find(getCMSAlgorithmName(cmsSignature));
		return sigAlg.getAlgorithm().getId();
	}

	public String getCMSIssuer(String cmsSignature)
			throws CMSException, NoSuchAlgorithmException, NoSuchProviderException, IOException {
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSSignedData signedData = new CMSSignedData(signature);
		SignerInformationStore signers = signedData.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		String Issuer = CerHelper.toStringReversed(X500Name.getInstance(signer.getSID().getIssuer().getEncoded()));
		Issuer = CorrectIssuerDN(Issuer);
		return Issuer;
	}

	public String getCertificate(String ldapServer, String ldapPort, String ldapCertSearchDN, String ldp_usr,
			String ldp_pass, String cer_issuer, String cer_serial) throws CertificateException, NamingException {
		// Date d1 = new Date(System.currentTimeMillis());
		X509Certificate cer = getCertificateX509(ldapServer, ldapPort, ldapCertSearchDN, ldp_usr, ldp_pass, cer_issuer,
				cer_serial);
		if (cer == null)
			return "";

		// Date d2 = new Date(System.currentTimeMillis());
		// long diffInMillies = Math.abs(d2.getTime() - d1.getTime());
		// System.out.println(diffInMillies);

		/* sertifikat tapılıbsa base64-ə çeviririk */
		return Base64Utils.base64Encode(cer.getEncoded());
	}

	// bax cer_id artiqdir
	public String getCertificate(String cer_issuer, String cer_serial, String cer_id)
			throws CertificateEncodingException, CertificateException, NamingException, ParserConfigurationException,
			SAXException, IOException {
		ConfigClass conf = LoadConfig(cer_id);
		return getCertificate(conf.LdapServer, conf.LdapPort, conf.LdapCertSearchDN, conf.LdapUser, conf.LdapPassword,
				cer_issuer, cer_serial);
	}

	public String getCertificate(String cer_url)
			throws FileNotFoundException, CertificateException, CRLException, IOException {
		X509Certificate c509 = null;
		String x509 = null;
		String beginCert = "-----BEGIN CERTIFICATE-----";
		String endCert = "-----END CERTIFICATE-----";
		if (cer_url.startsWith("http://") || cer_url.startsWith("https://") || cer_url.startsWith("ftp://")) {
			c509 = CerHelper.downloadCerFromWeb(cer_url);
		} else
			c509 = CerHelper.readCertFromFile(cer_url);
		x509 = beginCert + Base64Utils.base64Encode(c509.getEncoded()) + endCert;
		return x509;
	}

	public String getCertificate(String cer_serial, String cer_id) throws CertificateEncodingException,
			CertificateException, NamingException, ParserConfigurationException, SAXException, IOException {
		ConfigClass conf = LoadConfig(cer_id);
		return getCertificate(conf.LdapServer, conf.LdapPort, conf.LdapCertSearchDN, conf.LdapUser, conf.LdapPassword,
				conf.CerIssuer, cer_serial);
	}

	public String ConvBlock4(String block4) throws Exception {
		String stringToSign = block4.replaceAll("\r\n", "\n");
		return new String(stringToSign.getBytes("UTF-16LE"));
	}

	public byte[] ConvBlock4(byte[] block4) throws Exception {
		String stringToSign = new String(block4).replaceAll("\r\n", "\n");
		return stringToSign.getBytes("UTF-16LE");
	}

	public boolean checkCerValidity(String certificate, Date val_date) throws CertificateException {
		X509Certificate x509 = getCertFromString(certificate);
		try {
			x509.checkValidity(val_date);
			return true;
		} catch (CertificateExpiredException ex) {
			return false;
		} catch (CertificateNotYetValidException ex) {
			return false;
		}
	}

	public boolean checkCerAuthority(String endcer, String authcer)
			throws CertificateException, InvalidKeyException, NoSuchAlgorithmException, NoSuchProviderException {
		X509Certificate x509endcer = getCertFromString(endcer);
		X509Certificate x509authcer = getCertFromString(authcer);
		try {
			x509endcer.verify(x509authcer.getPublicKey());
			return true;
		} catch (SignatureException ex) {
			return false;
		}
	}

	public boolean verifyCRLFromLDAP(String certificate, String ldapServer, String ldapPort, String ldapCertSearchDN,
			String ldp_usr, String ldp_pass, String cer_issuer)
			throws CertificateException, CRLException, NamingException {
		X509Certificate cert = getCertFromString(certificate);
		X509CRL crl = CRLHelper.downloadCRLFromLDAP(ldapServer, ldapPort, ldp_usr, ldp_pass, ldapCertSearchDN,
				cer_issuer);
		if (!crl.isRevoked(cert)) {
			return true;
		}
		return false;
	}

	public boolean verifyCRLFromWeb(String certificate, String URL)
			throws MalformedURLException, CertificateException, CRLException, IOException {
		X509Certificate cert = getCertFromString(certificate);
		X509CRL crl = CRLHelper.readCRLFromWeb(cert, URL);
		if (!crl.isRevoked(cert)) {
			return true;
		}
		return false;
	}

	public boolean verifyCRLFromFile(String certificate, String pathToFile)
			throws FileNotFoundException, CertificateException, CRLException, IOException {
		X509Certificate cert = getCertFromString(certificate);
		X509CRL crl = CRLHelper.readCRLFromFile(pathToFile);
		if (!crl.isRevoked(cert)) {
			return true;
		}
		return false;
	}

	/**
	 * XML strukturda olan məlumatın verilmiş parametrlər əsasında imzalanması
	 * 
	 * @param xmlDoc            XML strukturda verilmiş imzalanacaq string tipli
	 *                          məlumat
	 * @param nodeToSign        Verilmiş xmlDOC sənədinin hansı tagi imzalanacaqsa
	 *                          həmin tag-in adını bildirir.Nümunə:
	 *                          Document:Document
	 * @param hashOID           İstifadə olunan imzalama alqoritmi üçün təyin
	 *                          olunmuş beynəlxalq OID. Ala biləcəyi qiymətlər:
	 *                          https://msdn.microsoft.com/en-us/library/ff635603.aspx
	 *                          Nümunə:sha1RSA:1.2.840.113549.1.1.5;sha256RSA:1.2.840.113549.1.1.11;sha512RSA:1.2.840.113549.1.1.13
	 * @param cerIssuer
	 * @param cerSerial
	 * @param containerName
	 * @param containerPassword
	 * @param containerFilePath
	 * @return Verilmiş xmlDoc sənədində <AppHdr><Sgntr></Sgntr></AppHdr> taginin
	 *         içərisinə XADES-BES strukturunda <Signature> imza tagini əlavə edərək
	 *         həmin sənədi string formatında geri qaytarır
	 * @throws Exception
	 */
	public String SignXmlDocument(String xmlDoc, String nodeToSign, String cerIssuer, String cerSerial, String hashOID,
			String containerName, String containerPassword, String containerFilePath) throws Exception {
		final String xadesNS = "http://uri.etsi.org/01903/v1.3.2#";
		final String signedpropsIdSuffix = "-signedprops";
		final String providerName = System.getProperty("jsr105Provider", "org.jcp.xml.dsig.internal.dom.XMLDSigRI");

		XMLSignatureFactory fac = XMLSignatureFactory.getInstance("DOM",
				(Provider) Class.forName(providerName).newInstance());

		/* convert string to xmldocument */
		org.w3c.dom.Document doc = String2XmlDocument(xmlDoc);

		KeyStoreHolder storeHolder = getKeyStoreHolder(cerIssuer, cerSerial, containerName, containerPassword,
				containerFilePath);
		PrivateKey pk = null;

		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null && !containerPassword.isEmpty() && !containerPassword.equals(""))
				passw = containerPassword.toCharArray();

			pk = (PrivateKey) storeHolder.getStore().getKey(storeHolder.getAlias(), passw);
		}

		/* Prepare KeyInfo */
		KeyInfoFactory kif = fac.getKeyInfoFactory();
		X509IssuerSerial x509is = kif.newX509IssuerSerial(
				storeHolder.getCertificate().getIssuerX500Principal().toString(),
				storeHolder.getCertificate().getSerialNumber());
		X509Data x509data = kif.newX509Data(Collections.singletonList(x509is));
		final String keyInfoId = "_" + UUID.randomUUID().toString();
		KeyInfo ki = kif.newKeyInfo(Collections.singletonList(x509data), keyInfoId);

		/* Prepare references */
		List<Reference> refs = new ArrayList<>();

		Reference ref1 = fac.newReference("#" + keyInfoId,
				fac.newDigestMethod(AlgorithmIDf.getDigestMethodNameByHashOID(hashOID), null),
				Collections.singletonList(
						fac.newCanonicalizationMethod(CanonicalizationMethod.EXCLUSIVE, (XMLStructure) null)),
				null, null);
		refs.add(ref1);

		final String signedpropsId = "_" + UUID.randomUUID().toString() + signedpropsIdSuffix;
		Reference ref2 = fac.newReference("#" + signedpropsId,
				fac.newDigestMethod(AlgorithmIDf.getDigestMethodNameByHashOID(hashOID), null),
				Collections.singletonList(
						fac.newCanonicalizationMethod(CanonicalizationMethod.EXCLUSIVE, (XMLStructure) null)),
				"http://uri.etsi.org/01903/v1.3.2#SignedProperties", null);
		refs.add(ref2);

		Reference ref3 = fac.newReference(null,
				fac.newDigestMethod(AlgorithmIDf.getDigestMethodNameByHashOID(hashOID), null),
				Collections.singletonList(
						fac.newCanonicalizationMethod(CanonicalizationMethod.EXCLUSIVE, (XMLStructure) null)),
				null, null);
		refs.add(ref3);

		SignedInfo si = fac.newSignedInfo(
				fac.newCanonicalizationMethod(CanonicalizationMethod.EXCLUSIVE, (XMLStructure) null),
				fac.newSignatureMethod(AlgorithmIDf.getSignatureMethodNameByHashOID(hashOID), null), refs);

		/* Create element AppHdr/Sgntr that will contain the <ds:Signature> */
		org.w3c.dom.Node appHdr = null;
		org.w3c.dom.NodeList sgntrList = doc.getElementsByTagName("AppHdr");
		if (sgntrList.getLength() != 0) {
			appHdr = sgntrList.item(0);
		}

		if (appHdr == null) {
			throw new Exception("AppHdr tag-i tapılmadı");
		}

		org.w3c.dom.Node sgntr = appHdr.appendChild(doc.createElementNS(appHdr.getNamespaceURI(), "Sgntr"));

		DOMSignContext dsc = new DOMSignContext(pk, sgntr);
		System.out.println("certDetails.getPrivateKey()" + pk);
		dsc.putNamespacePrefix(XMLSignature.XMLNS, "ds");

		/*
		 * Set up <ds:Object> with <QualifiyingProperties> inside that includes
		 * SigningTime
		 */
		org.w3c.dom.Element QPElement = doc.createElementNS(xadesNS, "xades:QualifyingProperties");

		org.w3c.dom.Element SPElement = doc.createElementNS(xadesNS, "xades:SignedProperties");
		SPElement.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xades", xadesNS);
		SPElement.setAttributeNS(null, "Id", signedpropsId);
		dsc.setIdAttributeNS(SPElement, null, "Id");
		SPElement.setIdAttributeNS(null, "Id", true);
		QPElement.appendChild(SPElement);

		org.w3c.dom.Element SSPElement = doc.createElementNS(xadesNS, "xades:SignedSignatureProperties");
		SPElement.appendChild(SSPElement);

		final DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		// final DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
		String signingTime = df.format(new Date());

		org.w3c.dom.Element STElement = doc.createElementNS(xadesNS, "xades:SigningTime");
		STElement.appendChild(doc.createTextNode(signingTime));
		SSPElement.appendChild(STElement);

		DOMStructure qualifPropStruct = new DOMStructure(QPElement);

		List<DOMStructure> xmlObj = new ArrayList<>();
		xmlObj.add(qualifPropStruct);
		XMLObject object = fac.newXMLObject(xmlObj, null, null, null);

		List<XMLObject> objects = Collections.singletonList(object);

		// Set up custom URIDereferencer to process Reference without URI.
		// This Reference points to element <Document> of MX message
		final org.w3c.dom.NodeList docNodes = doc.getElementsByTagName(nodeToSign);
		final org.w3c.dom.Node docNode = docNodes.item(0);

		java.io.ByteArrayOutputStream refOutputStream = new java.io.ByteArrayOutputStream();
		Transformer xform = TransformerFactory.newInstance().newTransformer();
		xform.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
		xform.transform(new DOMSource(docNode), new StreamResult(refOutputStream));
		InputStream refInputStream = new ByteArrayInputStream(refOutputStream.toByteArray());
		dsc.setURIDereferencer(new com.crypto.utils.helpers.NoUriDereferencer(refInputStream));

		// sign it!
		XMLSignature signature = fac.newXMLSignature(si, ki, objects, null, null);
		signature.sign(dsc);

		return Xml2String(doc);
	}

	/**
	 * XML strukturda olan məlumatın tələb olunan rekvizitlər üçün parametr faylında
	 * müəyyənləşdirilmiş qiymətlər əsasında imzalanması. Parametr faylın strukturu
	 * 
	 * @param xmlDoc     XML strukturda verilmiş imzalanacaq string tipli məlumat
	 * @param nodeToSign Verilmiş xmlDOC sənədinin hansı tagi imzalanacaqsa həmin
	 *                   tag-in adını bildirir.Nümunə: Document:Document
	 * @param cerId      Konfiqurasiya faylında sertifikat parametrlərinin
	 *                   yerləşdiyi <certificate> tag-inin İD nömrəsini
	 *                   bildirir(Əlavə 1).Ala biləcəyi qiymətlər: 01- 1 nömrəli
	 *                   sertifikat parametrləri. 0n- n nömrəli sertifikat
	 *                   parametrləri.
	 * @return Verilmiş xmlDoc sənədində <AppHdr><Sgntr></Sgntr></AppHdr> taginin
	 *         içərisinə XADES-BES strukturunda <Signature> imza tagini əlavə edərək
	 *         həmin sənədi string formatında geri qaytarır
	 */
	public String SignXMLDocument(String xmlDoc, String nodeToSign, String cerId) throws Exception {
		ConfigClass conf = LoadConfig("");
		return SignXmlDocument(xmlDoc, nodeToSign, conf.CerIssuer, conf.CerSerial, conf.HashOid, conf.ContainerName,
				conf.ContainerPassword, conf.ContainerFilePath);
	}

	/**
	 * Bu funksiyada XML strukturlu sənədin XADES tipli imzasının yoxlanılması üçün
	 * istifadə olunacaq sertifikatın parametr faylı əsasında LDAP-dan götürülməsi
	 * nəzərdə tutulur.Parametr faylının strukturu Əlavə 1-də verilmişdir.
	 * Sertifikatın tapılması üçün issuer və serial parametrləri Signature tag-ində
	 * verilir.
	 * 
	 * @param signedXmlDoc     İmzası yoxlanılacaq XML sənəd
	 * @param signedNode       Verilmiş XML sənəddə hansı tag-in imzası yoxlanacaqsa
	 *                         həmin tag-i bildirir. Nümunə: Document:Document
	 * @param ldapServer       Elektron imzanı yoxlamaq üçün lazım olan sertifikatın
	 *                         əldə edilməsi məqsədilə istifadə olunacaq LDAP
	 *                         serverin ünvanı. Nümunə: ldap.bsxm.cbar.az
	 * @param ldapPort         LDAP direktoriyasının portunu bildirir. Nümunə: 389
	 * @param ldapCertSearchDN Elektron imzanı yoxlamaq üçün lazım olan sertifikatın
	 *                         axtarılacağı direktoriyanı bildirir. Nümunə: CN=
	 *                         users, CN= pki, DC= bsxm, DC= cbar, DC= az
	 * @param ldapUser         LDAP serverə qoşulmaq üçün təqdim edilən istifadəçi
	 *                         adı
	 * @param ldapPassword     İmzanın yoxlanılmasının nəticəsi string formatında
	 *                         geri qaytarılır. İmza təsdiqlənirsə “verified” əks
	 *                         halda ”not verified” cavabı qaytarılır
	 * @return İmzanın yoxlanılmasının nəticəsi string formatında geri qaytarılır.
	 *         İmza təsdiqlənirsə “verified” əks halda ”not verified” cavabı
	 *         qaytarılır
	 */
	public String VerifyXmlDocument(String signedXmlDoc, String signedNode, String ldapServer, String ldapPort,
			String ldapCertSearchDN, String ldapUser, String ldapPassword) throws Exception {
		org.w3c.dom.Document doc = null;
		try {
			doc = String2XmlDocument(signedXmlDoc);
		} catch (Exception e) {
			throw new Exception("signedXmlDoc formatı düzgün deyil");
		}

		XPath xpath = XPathFactory.newInstance().newXPath();
		String xpathExpression = "//*[local-name()='Signature']";
		org.w3c.dom.NodeList nodes = (org.w3c.dom.NodeList) xpath.evaluate(xpathExpression, doc.getDocumentElement(),
				XPathConstants.NODESET);

		if (nodes == null || nodes.getLength() == 0) {
			throw new Exception("Sənəddə imza tapılmadı");
		}
		org.w3c.dom.Node nodeSignature = nodes.item(0);

		org.w3c.dom.NodeList nlSerialNumber = doc.getElementsByTagName("ds:X509SerialNumber");
		if ((nlSerialNumber == null) || (nlSerialNumber.getLength() == 0)) {
			throw new Exception("X509SerialNumber tag-i tapılmadı");
		}
		String serialNumberStr = nlSerialNumber.item(0).getTextContent();
		String serialNumber = getSerial(serialNumberStr);

		org.w3c.dom.NodeList nlIssuer = doc.getElementsByTagName("ds:X509IssuerName");
		if ((nlIssuer == null) || (nlIssuer.getLength() == 0)) {
			throw new Exception("X509IssuerName tag-i tapılmadı");
		}
		String issuer = nlIssuer.item(0).getTextContent();

		X509Certificate cert = getCertificateX509(ldapServer, ldapPort, ldapCertSearchDN, ldapUser, ldapPassword, issuer, serialNumber);
		if (cert == null) {
			throw new Exception("Sertifikat LDAP direktoriyasında tapılmadı");
		}

		final KeySelector mockKeySelector = new KeySelector() {

			@Override
			public KeySelectorResult select(KeyInfo keyInfo, Purpose purpose, AlgorithmMethod method,
					XMLCryptoContext context) throws KeySelectorException {

				return new KeySelectorResult() {
					@Override
					public Key getKey() {
						return cert.getPublicKey();
					}
				};
			}
		};

		XMLSignatureFactory fac = XMLSignatureFactory.getInstance("DOM");
		DOMValidateContext valContext = new DOMValidateContext(mockKeySelector, nodeSignature);

		// Set up custom URIDereferencer to process Reference without URI.
		// This Reference points to element <Document> of MX message
		final org.w3c.dom.NodeList docNodes = doc.getElementsByTagName(signedNode);
		final org.w3c.dom.Node docNode = docNodes.item(0);

		java.io.ByteArrayOutputStream refOutputStream = new java.io.ByteArrayOutputStream();
		Transformer xform = TransformerFactory.newInstance().newTransformer();
		xform.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
		xform.transform(new DOMSource(docNode), new StreamResult(refOutputStream));
		InputStream refInputStream = new ByteArrayInputStream(refOutputStream.toByteArray());
		valContext.setURIDereferencer(new com.crypto.utils.helpers.NoUriDereferencer(refInputStream));

		// Java 1.7.0_25+ complicates validation of
		// ds:Object/QualifyingProperties/SignedProperties
		// See details at https://bugs.openjdk.java.net/browse/JDK-8019379
		//
		// One of the solutions is to register the Id attribute using the
		// DOMValidateContext.setIdAttributeNS
		// method before validating the signature
		// NodeList nl = doc.getElementsByTagNameNS("xades", "SignedProperties");
		org.w3c.dom.NodeList nl = doc.getElementsByTagNameNS("http://uri.etsi.org/01903/v1.3.2#", "SignedProperties");

		if (nl.getLength() == 0) {
			throw new Exception("Sənəddə imza tapılmadı");
		}
		org.w3c.dom.Element elemSignedProps = (org.w3c.dom.Element) nl.item(0);
		valContext.setIdAttributeNS(elemSignedProps, null, "Id");
		XMLSignature signature = fac.unmarshalXMLSignature(valContext);

		boolean signatureIsValid = false;
		try {

			signatureIsValid = signature.validate(valContext);
		} catch (Exception e) {
		}

		if (signatureIsValid)
			return "verified";
		else
			return "not verified";
	}

	/**
	 * Bu funksiyada XML strukturlu sənədin XADES tipli imzasının yoxlanılması üçün
	 * istifadə olunan sertifikatın parametr olaraq funksiyaya ötürülən LDAP
	 * rekvizitləri əsasında müəyyənləşdirilməsi nəzərdə tutulur.
	 * 
	 * @param signedXmlDoc İmzası yoxlanılacaq XML sənəd.
	 * @param signedNode   Verilmiş XML sənəddə hansı tag-in imzası yoxlanacaqsa
	 *                     həmin tag-i bildirir. Nümunə: Document:Document
	 * @return İmzanın yoxlanılmasının nəticəsi string formatında geri qaytarılır.
	 *         İmza təsdiqlənirsə “verified” əks halda ”not verified” cavabı
	 *         qaytarılır
	 * @throws Exception
	 */
	public String VerifyXmlDocument(String signedXmlDoc, String signedNode) throws Exception {
		ConfigClass conf = LoadConfig("");
		return VerifyXmlDocument(signedXmlDoc, signedNode, conf.LdapServer, conf.LdapPort, conf.LdapCertSearchDN,
				conf.LdapUser, conf.LdapPassword);
	}

	/**
	 * byte[] tipli məlumatın verilmiş parametrlər əsasında imzalanması
	 * 
	 * @param inputData         İmzalanacaq məlumat
	 * @param cerIssuer         Məlumatı imzalamaq üçün istifadə olunacaq
	 *                          kriptoqrafik açara uyğun sertifikatı verən orqanın
	 *                          tam adnı bildirir. Nümunə: ”CN = ARMB BSXM,O =
	 *                          Azərbaycan Respublikasının Mərkəzi Bankı,OU =Bank
	 *                          Sertifikat Xidmətləri Mərkəzi,E = e-imza@cbar.az,L =
	 *                          Bakı,C = AZ”
	 * @param cerSerial         Məlumatı imzalamaq üçün istifadə olunacaq
	 *                          kriptoqrafik açara uyğun sertifikatın serial
	 *                          nömrəsini bildirir. Nümunə: ”00 e3 ad da af f4 c6 d7
	 *                          b8 7b 78 06 3e 67 a8 0c bb”
	 * @param hashOID           İstifadə olunan imzalama alqoritmi üçün təyin
	 *                          olunmuş beynəlxalq OID. Ala biləcəyi qiymətlər:
	 *                          https://msdn.microsoft.com/en-us/library/ff635603.aspx
	 *                          Nümunə:sha1RSA:1.2.840.113549.1.1.5;sha256RSA:1.2.840.113549.1.1.11;
	 *                          sha512RSA:1.2.840.113549.1.1.13
	 * @param containerName     İmza qoyacaq sertifikatın yerləşdiyi konteynerin
	 *                          tipi. Ala biləcəyi qiymətlər: WINDOWS- Windows
	 *                          əməliyyat sisteminin sertifikat konteyneri. PKCS11 –
	 *                          pkcs11 tipli smartcard və ya eTokendə olan
	 *                          sertifikat konteyneri PFX_FILE - .pfx faylında
	 *                          yerləşən sertifikat konteyneri JKS_FILE - java
	 *                          keystore tipli .jks faylında yerləşən sertifikat
	 *                          konteyneri
	 * @param containerPassword containerName parametrində göstərilən konteynerin
	 *                          şifrəsi. PKCS11 tipi üçün şifrə Smartcard və ya
	 *                          eTokenin PİN kodu, PFX_FİLE tipi üçün .pfx faylın
	 *                          şifrəsi, JKS_FİLE tipi üçün .jks faylın şifrəsi
	 *                          WINDOWS tipində isə null olacaqdır
	 * @param containerFilePath .pfx/.jks tipli sertifikat faylının və ya PKCS11
	 *                          tipli konteynerlərdə kriptoqrafik kitabxana faylının
	 *                          adının bildirir.
	 * @param detached          İmzalanacaq məlumatın PKCS7 formatına daxil edilib
	 *                          edilməyəcəyini bildirir.Ala biləcəyi qiymətlər: 1-
	 *                          Məlumat daxil ediləcək. 0-Məlumat daxil edilməyəcək.
	 * @return PKCS7 formatında olan elektron imza məlumatının Base64
	 *         kodlaşdırmasında olan string tipli dəyəri
	 * @throws KeyStoreException
	 * @throws GeneralSecurityException
	 * @throws IOException
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws OperatorCreationException
	 * @throws CMSException
	 */
	public String signTransportData(byte[] inputData, String cerIssuer, String cerSerial, String hashOID,
			String containerName, String containerPassword, String containerFilePath, int detached)
			throws KeyStoreException, GeneralSecurityException, IOException, SecurityException,
			IllegalArgumentException, ClassNotFoundException, NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException, OperatorCreationException, CMSException {
		KeyStoreHolder storeHolder = getKeyStoreHolder(cerIssuer, cerSerial, containerName, containerPassword,
				containerFilePath);
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null && !containerPassword.isEmpty() && !containerPassword.equals(""))
				passw = containerPassword.toCharArray();
			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(storeHolder.getAlias(), passw);
			return Base64Utils.base64Encode(sign(inputData, hashOID, pk, storeHolder, detached));
		}
		return null;
	}

	/**
	 * 
	 * @param inputData         İmzalanacaq məlumat
	 * @param cerIssuer         Məlumatı imzalamaq üçün istifadə olunacaq
	 *                          kriptoqrafik açara uyğun sertifikatı verən orqanın
	 *                          tam adnı bildirir. Nümunə: ”CN = ARMB BSXM,O =
	 *                          Azərbaycan Respublikasının Mərkəzi Bankı,OU =Bank
	 *                          Sertifikat Xidmətləri Mərkəzi,E = e-imza@cbar.az,L =
	 *                          Bakı,C = AZ”
	 * @param cerSerial         Məlumatı imzalamaq üçün istifadə olunacaq
	 *                          kriptoqrafik açara uyğun sertifikatın serial
	 *                          nömrəsini bildirir. Nümunə: ”00 e3 ad da af f4 c6 d7
	 *                          b8 7b 78 06 3e 67 a8 0c bb”
	 * @param hashOID           İstifadə olunan imzalama alqoritmi üçün təyin
	 *                          olunmuş beynəlxalq OID. Ala biləcəyi qiymətlər:
	 *                          https://msdn.microsoft.com/en-us/library/ff635603.aspx
	 *                          Nümunə:sha1RSA:1.2.840.113549.1.1.5;sha256RSA:1.2.840.113549.1.1.11;
	 *                          sha512RSA:1.2.840.113549.1.1.13
	 * @param containerName     İmza qoyacaq sertifikatın yerləşdiyi konteynerin
	 *                          tipi. Ala biləcəyi qiymətlər: WINDOWS- Windows
	 *                          əməliyyat sisteminin sertifikat konteyneri. PKCS11 –
	 *                          pkcs11 tipli smartcard və ya eTokendə olan
	 *                          sertifikat konteyneri PFX_FILE - .pfx faylında
	 *                          yerləşən sertifikat konteyneri JKS_FILE - java
	 *                          keystore tipli .jks faylında yerləşən sertifikat
	 *                          konteyneri
	 * @param containerPassword containerName parametrində göstərilən konteynerin
	 *                          şifrəsi. PKCS11 tipi üçün şifrə Smartcard və ya
	 *                          eTokenin PİN kodu, PFX_FİLE tipi üçün .pfx faylın
	 *                          şifrəsi, JKS_FİLE tipi üçün .jks faylın şifrəsi
	 *                          WINDOWS tipində isə null olacaqdır
	 * @param containerFilePath .pfx/.jks tipli sertifikat faylının və ya PKCS11
	 *                          tipli konteynerlərdə kriptoqrafik kitabxana faylının
	 *                          adının bildirir.
	 * @param detached          İmzalanacaq məlumatın PKCS7 formatına daxil edilib
	 *                          edilməyəcəyini bildirir.Ala biləcəyi qiymətlər: 1-
	 *                          Məlumat daxil ediləcək. 0-Məlumat daxil edilməyəcək.
	 * @return PKCS7 formatında olan elektron imza məlumatının Base64
	 *         kodlaşdırmasında olan string tipli dəyəri
	 * @throws KeyStoreException
	 * @throws GeneralSecurityException
	 * @throws IOException
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws OperatorCreationException
	 * @throws CMSException
	 */
	public String signTransportData(String inputData, String cerIssuer, String cerSerial, String hashOID,
			String containerName, String containerPassword, String containerFilePath, int detached)
			throws KeyStoreException, GeneralSecurityException, IOException, SecurityException,
			IllegalArgumentException, ClassNotFoundException, NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException, OperatorCreationException, CMSException {

		byte[] inputDataBytes = inputData.getBytes();
		return signTransportData(inputDataBytes, cerIssuer, cerSerial, hashOID, containerName, containerPassword,
				containerFilePath, detached);
	}

	/**
	 * String tipli məlumatın verilmiş parametrlər əsasında imzalanması
	 * 
	 * @param inputData İmzalanacaq məlumat
	 * @param cerId     İmzalanacaq məlumatın PKCS7 formatına daxil edilib
	 *                  edilməyəcəyini bildirir. Ala biləcəyi qiymətlər: 1- Məlumat
	 *                  daxil ediləcək. 0-Məlumat daxil edilməyəcək
	 * @return PKCS7 formatında olan elektron imza məlumatının Base64
	 *         kodlaşdırmasında olan string tipli dəyəri
	 * @throws KeyStoreException
	 * @throws GeneralSecurityException
	 * @throws IOException
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws OperatorCreationException
	 * @throws CMSException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 */
	public String signTransportData(String inputData, String cerId) throws KeyStoreException, GeneralSecurityException,
			IOException, SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException, InvocationTargetException, OperatorCreationException,
			CMSException, ParserConfigurationException, SAXException {

		return signTransportData(inputData.getBytes(), cerId);
	}

	/**
	 * byte[] tipli məlumatın verilmiş parametrlər əsasında imzalanması
	 * 
	 * @param inputData İmzalanacaq məlumat
	 * @param cerId     İmzalanacaq məlumatın PKCS7 formatına daxil edilib
	 *                  edilməyəcəyini bildirir. Ala biləcəyi qiymətlər: 1- Məlumat
	 *                  daxil ediləcək. 0-Məlumat daxil edilməyəcək
	 * @return PKCS7 formatında olan elektron imza məlumatının Base64
	 *         kodlaşdırmasında olan string tipli dəyəri
	 * @throws KeyStoreException
	 * @throws GeneralSecurityException
	 * @throws IOException
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws OperatorCreationException
	 * @throws CMSException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 */
	public String signTransportData(byte[] inputData, String cerId) throws KeyStoreException, GeneralSecurityException,
			IOException, SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException, InvocationTargetException, OperatorCreationException,
			CMSException, ParserConfigurationException, SAXException {

		ConfigClass conf = LoadConfig(cerId);
		return signTransportData(inputData, conf.CerIssuer, conf.CerSerial, conf.HashOid, conf.ContainerName,
				conf.ContainerPassword, conf.ContainerFilePath, conf.Detached);
	}

	/**
	 * Bu funksiyada String tipli məlumatın elektron imzasının yoxlanılması üçün
	 * istifadə olunacaq sertifikatın parametr faylı əsasında LDAP-dan götürülməsi
	 * nəzərdə tutulur. Parametr faylının strukturu Əlavə 1-də verilmişdir
	 * 
	 * @param inputData        İmzası yoxlanılacaq məlumat
	 * @param cmsSignature     PKCS7 formatında elektron imza məlumatının Base64
	 *                         kodlaşdırmasında olan string tipli dəyəri
	 * @param ldapServer       Elektron imzanı yoxlamaq üçün lazım olan sertifikatın
	 *                         əldə edilməsi məqsədilə istifadə olunacaq LDAP
	 *                         serverin ünvanı. Nümunə: ldap.bsxm.cbar.az
	 * @param ldapPort         LDAP direktoriyasının portunu bildirir. Nümunə: 389
	 * @param ldapCertSearchDN Elektron imzanı yoxlamaq üçün lazım olan sertifikatın
	 *                         axtarılacağı direktoriyanı bildirir. Nümunə:
	 *                         CN=users,CN=pki, DC=bsxm, DC=cbar,DC=az
	 * @param ldapUser         LDAP serverə qoşulmaq üçün təqdim edilən istifadəçi
	 *                         adı
	 * @param ldapPassword     LDAP serverə qoşulmaq üçün təqdim edilən
	 *                         istifadəçinin şifrəsi
	 * @return İmzanın yoxlanılmasının nəticəsi string formatında geri qaytarılır.
	 *         İmza təsdiqlənirsə “verified” əks halda ”not verified” cavabı
	 *         qaytarılır
	 * @throws CMSException
	 * @throws OperatorCreationException
	 * @throws CertificateException
	 * @throws NamingException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public String verifyTransportData(String inputData, String cmsSignature, String ldapServer, String ldapPort,
			String ldapCertSearchDN, String ldapUser, String ldapPassword)
			throws CMSException, OperatorCreationException, CertificateException, NamingException,
			ParserConfigurationException, SAXException, IOException {
		byte[] inputDataBytes = inputData.getBytes();
		return verifyTransportData(inputDataBytes, cmsSignature, ldapServer, ldapPort, ldapCertSearchDN, ldapUser,
				ldapPassword);
	}

	/**
	 * Bu funksiyada byte[] tipli məlumatın elektron imzasının yoxlanılması üçün
	 * istifadə olunacaq sertifikatın parametr faylı əsasında LDAP-dan götürülməsi
	 * nəzərdə tutulur. Parametr faylının strukturu Əlavə 1-də verilmişdir
	 * 
	 * @param inputData        İmzası yoxlanılacaq məlumat
	 * @param cmsSignature     PKCS7 formatında elektron imza məlumatının Base64
	 *                         kodlaşdırmasında olan string tipli dəyəri
	 * @param ldapServer       Elektron imzanı yoxlamaq üçün lazım olan sertifikatın
	 *                         əldə edilməsi məqsədilə istifadə olunacaq LDAP
	 *                         serverin ünvanı. Nümunə: ldap.bsxm.cbar.az
	 * @param ldapPort         LDAP direktoriyasının portunu bildirir. Nümunə: 389
	 * @param ldapCertSearchDN Elektron imzanı yoxlamaq üçün lazım olan sertifikatın
	 *                         axtarılacağı direktoriyanı bildirir. Nümunə:
	 *                         CN=users,CN=pki, DC=bsxm, DC=cbar,DC=az
	 * @param ldapUser         LDAP serverə qoşulmaq üçün təqdim edilən istifadəçi
	 *                         adı
	 * @param ldapPassword     LDAP serverə qoşulmaq üçün təqdim edilən
	 *                         istifadəçinin şifrəsi
	 * @return İmzanın yoxlanılmasının nəticəsi string formatında geri qaytarılır.
	 *         İmza təsdiqlənirsə “verified” əks halda ”not verified” cavabı
	 *         qaytarılır
	 * @throws CMSException
	 * @throws OperatorCreationException
	 * @throws CertificateException
	 * @throws NamingException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public String verifyTransportData(byte[] inputData, String cmsSignature, String ldapServer, String ldapPort,
			String ldapCertSearchDN, String ldapUser, String ldapPassword)
			throws CMSException, OperatorCreationException, CertificateException, NamingException,
			ParserConfigurationException, SAXException, IOException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputData);
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();
		// String ldapUser2 = ldapUser;
		// String ldapPassword2 = ldapPassword;
		// String Issuer = signerId.getIssuer().toString();
		String Issuer = CerHelper.toStringReversed(X500Name.getInstance(signerId.getIssuer().getEncoded()));
		Issuer = CorrectIssuerDN(Issuer);// Issuer.replaceAll(",", ", ");
		String hexString = signerId.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();
		while ((sb.length() + hexString.length()) < (2 * signerId.getSerialNumber().toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		String serialNumber = sb.toString();

		X509Certificate cert = getCertificateX509(ldapServer, ldapPort, ldapCertSearchDN, ldapUser, ldapPassword,
				Issuer, serialNumber);
		boolean verified = signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert));
		if (verified)
			return "verified";
		else
			return "not verified";
	}

	/**
	 * Bu funksiyada byte[] tipli məlumatın elektron imzasının yoxlanılması üçün
	 * istifadə olunacaq sertifikatın signercer parametri vasitəsilə funksiyaya
	 * ötürülməsi nəzərdə tutulur
	 * 
	 * @param inputData    İmzası yoxlanılacaq məlumat
	 * @param cmsSignature PKCS7 formatında elektron imza məlumatının Base64
	 *                     kodlaşdırmasında olan string tipli dəyəri
	 * @param signerCer    İmzanın yoxlanılması üçün istifadə olunacaq
	 *                     X509Certificate obyektinin Base64String forması
	 * @return İmzanın yoxlanılmasının nəticəsi string formatında geri qaytarılır.
	 *         İmza təsdiqlənirsə “verified” əks halda ”not verified” cavabı
	 *         qaytarılır
	 * @throws CMSException
	 * @throws OperatorCreationException
	 * @throws CertificateException
	 * @throws KeyStoreException
	 */
	public String verifyTransportData(byte[] inputData, String cmsSignature, String signerCer)
			throws CMSException, OperatorCreationException, CertificateException, KeyStoreException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputData);
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		X509Certificate cert = getCertFromString(signerCer);
		Security.addProvider(new BouncyCastleProvider());
		boolean verified = signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert));
		if (verified)
			return "verified";
		else
			return "not verified";
	}

	/**
	 * Bu funksiyada string tipli məlumatın elektron imzasının yoxlanılması üçün
	 * istifadə olunacaq sertifikatın signercer parametri vasitəsilə funksiyaya
	 * ötürülməsi nəzərdə tutulur
	 * 
	 * @param inputData    İmzası yoxlanılacaq məlumat
	 * @param cmsSignature PKCS7 formatında elektron imza məlumatının Base64
	 *                     kodlaşdırmasında olan string tipli dəyəri
	 * @param signerCer    İmzanın yoxlanılması üçün istifadə olunacaq
	 *                     X509Certificate obyektinin Base64String forması
	 * @return İmzanın yoxlanılmasının nəticəsi string formatında geri qaytarılır.
	 *         İmza təsdiqlənirsə “verified” əks halda ”not verified” cavabı
	 *         qaytarılır
	 * @throws CMSException
	 * @throws OperatorCreationException
	 * @throws CertificateException
	 * @throws KeyStoreException
	 */
	public String verifyTransportData(String inputData, String cmsSignature, String signerCer)
			throws CMSException, OperatorCreationException, CertificateException, KeyStoreException {
		byte[] inputDataBytes = inputData.getBytes();
		return verifyTransportData(inputDataBytes, cmsSignature, signerCer);
	}

	/**
	 * Bu funksiyada byte[] tipli məlumatın elektron imzasının yoxlanılması üçün
	 * istifadə olunacaq sertifikatın parametr faylı əsasında LDAP-dan götürülməsi
	 * nəzərdə tutulur. Parametr faylının strukturu Əlavə 1-də verilmişdir
	 * 
	 * @param inputData    İmzası yoxlanılacaq məlumat
	 * @param cmsSignature PKCS7 formatında elektron imza məlumatının Base64
	 *                     kodlaşdırmasında olan string tipli dəyəri
	 * @return İmzanın yoxlanılmasının nəticəsi string formatında geri qaytarılır.
	 *         İmza təsdiqlənirsə “verified” əks halda ”not verified” cavabı
	 *         qaytarılır
	 * @throws CMSException
	 * @throws OperatorCreationException
	 * @throws CertificateException
	 * @throws NamingException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public String verifyTransportData(String inputData, String cmsSignature)
			throws CMSException, OperatorCreationException, CertificateException, NamingException,
			ParserConfigurationException, SAXException, IOException {
		byte[] inputDataBytes = inputData.getBytes();
		return verifyTransportData(inputDataBytes, cmsSignature);
	}

	/**
	 * 
	 * @param inputData    İmzası yoxlanılacaq məlumat
	 * @param cmsSignature PKCS7 formatında elektron imza məlumatının Base64
	 *                     kodlaşdırmasında olan string tipli dəyəri
	 * @return İmzanın yoxlanılmasının nəticəsi string formatında geri qaytarılır.
	 *         İmza təsdiqlənirsə “verified” əks halda ”not verified” cavabı
	 *         qaytarılır
	 * @throws CMSException
	 * @throws OperatorCreationException
	 * @throws CertificateException
	 * @throws NamingException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public String verifyTransportData(byte[] inputData, String cmsSignature)
			throws CMSException, OperatorCreationException, CertificateException, NamingException,
			ParserConfigurationException, SAXException, IOException {
		ConfigClass conf = LoadConfig("");
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputData);
		CMSSignedData s = new CMSSignedData(typedData, signature);

		SignerInformationStore signers = s.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();

		// String Issuer = signerId.getIssuer().toString();
		String Issuer = CerHelper.toStringReversed(X500Name.getInstance(signerId.getIssuer().getEncoded()));
		Issuer = CorrectIssuerDN(Issuer);// Issuer.replaceAll(",", ", ");
		String hexString = signerId.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();
		while ((sb.length() + hexString.length()) < (2 * signerId.getSerialNumber().toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		String serialNumber = sb.toString();

		X509Certificate cert = getCertificateX509(conf.LdapServer, conf.LdapPort, conf.LdapCertSearchDN, conf.LdapUser,
				conf.LdapPassword, Issuer, serialNumber);

		Security.addProvider(new BouncyCastleProvider());
		boolean verified = signer.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert));

		if (verified)
			return "verified";
		else
			return "not verified";
	}

	// ****************************************************************
	// PUBLIC METHODS END *
	// ****************************************************************

	// ****************************************************************
	// PRIVATE METHODS BEGIN *
	// ****************************************************************

	public ConfigClass LoadConfig(String cerId) {
		ConfigClass res = new ConfigClass();

		Map xmlMap;
		try {
			xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);

			res.LdapServer = (String) xmlMap.get("ldapServer");
			res.LdapPort = (String) xmlMap.get("ldapPort");
			res.LdapUser = (String) xmlMap.get("ldapUser");
			res.LdapPassword = (String) xmlMap.get("ldapPassword");
			res.LdapCertSearchDN = (String) xmlMap.get("ldapCertSearchDN");

			res.SaveLdapSearchResult = ((String) xmlMap.get("saveLdapSearchResult")).equals("1");
			res.LdapSearchContainerName = (String) xmlMap.get("ldapSearchContainerName");
			res.LdapSearchContainerFilePath = (String) xmlMap.get("ldapSearchContainerFilePath");
			res.LdapSearchContainerPassword = (String) xmlMap.get("ldapSearchContainerPassword");

			if (cerId != "") {
				Map xmlCertMap = (Map) xmlMap.get(cerId);
				res.CerIssuer = (String) xmlCertMap.get("cer_issuer");
				res.CerSerial = (String) xmlCertMap.get("cer_serial");
				res.HashOid = (String) xmlCertMap.get("hash_oid");
				res.ContainerName = (String) xmlCertMap.get("containerName");
				res.ContainerPassword = (String) xmlCertMap.get("containerPassword");
				res.ContainerFilePath = (String) xmlCertMap.get("containerFilePath");
				res.Detached = Integer.parseInt((String) xmlCertMap.get("detached"));
			}

		} catch (Exception e) {
			// e.printStackTrace();
		}
		return res;
	}

	private X509Certificate getCertificateX509(String ldapServer, String ldapPort, String ldapCertSearchDN,
			String ldp_usr, String ldp_pass, String cer_issuer, String cer_serial)
			throws CertificateException, NamingException {
		ConfigClass conf = LoadConfig("");

		String correctedIssuer = CorrectIssuerDN(cer_issuer);
		
		boolean searchInContainer = conf.SaveLdapSearchResult;
		/*
		 * ( (conf.LdapSearchContainerName != null) &&
		 * (!conf.LdapSearchContainerName.isEmpty()) && (
		 * (conf.LdapSearchContainerName.equals(WINDOWS_CONTAINER_NAME)) || (
		 * (conf.LdapSearchContainerName.equals(JKS_CONTAINER_NAME)) &&
		 * (conf.LdapSearchContainerFilePath != null) &&
		 * (!conf.LdapSearchContainerFilePath.isEmpty()) &&
		 * (conf.LdapSearchContainerPassword != null) &&
		 * (!conf.LdapSearchContainerPassword.isEmpty()) ) ) );
		 */

		try {
			if (!searchInContainer) {
				X509Certificate cerInLdap = LdapReader.getCertificate(ldapServer, ldapPort, ldp_usr, ldp_pass,
						ldapCertSearchDN, cer_serial, correctedIssuer);
				return cerInLdap;
			} else {
				/* Əvvəlcə lokal konteynerdə axtarırıq */
				KeyStoreHolder storeHolder = getKeyStoreHolder(correctedIssuer, cer_serial, conf.LdapSearchContainerName,
						conf.LdapSearchContainerPassword, conf.LdapSearchContainerFilePath);

				if (storeHolder != null) {
					X509Certificate cerInLocal = storeHolder.getCertificate();
					if (cerInLocal != null)
						return cerInLocal;
				}

				/* Lokalda tapmasaq ldap direktoriyada axtaririq */
				X509Certificate cerInLdap = LdapReader.getCertificate(ldapServer, ldapPort, ldp_usr, ldp_pass,
						ldapCertSearchDN, cer_serial, correctedIssuer);

				/* Əgər ldap-da sertifikat yoxdursa null qaytarırıq */
				if (cerInLdap == null)
					return null;

				/* Ldap direktoriyadan tapdığımız sertifikatı lokal konteynerə yükləyirik */
				installCertificateToStore(conf.LdapSearchContainerName, conf.LdapSearchContainerFilePath,
						conf.LdapSearchContainerPassword, cerInLdap);

				/* Yenidən sertifikatı lokal konteynerdən axtarıb geri qaytarırıq */
				KeyStoreHolder storeHolderNewInstalled = getKeyStoreHolder(correctedIssuer, cer_serial,
						conf.LdapSearchContainerName, conf.LdapSearchContainerPassword,
						conf.LdapSearchContainerFilePath);
				if (storeHolderNewInstalled != null) {
					X509Certificate cerInLocalNewInstalled = storeHolderNewInstalled.getCertificate();
					if (cerInLocalNewInstalled != null)
						return cerInLocalNewInstalled;
				}
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}

		/* Ən sonda null qaytaririq */
		return null;

	}

	@SuppressWarnings("rawtypes")
	private byte[] decrypt(final String encodedCmss, PrivateKey key, KeyStoreHolder sHolder)
			throws CMSException, IOException, NoSuchProviderException, KeyStoreException {
		byte[] encodedCms = Base64Utils.base64Decode(encodedCmss);
		Security.addProvider(new com.sun.crypto.provider.SunJCE());

		// Security.addProvider(KeyStore.getInstance("Windows-my").getProvider());

		// CMSEnvelopedDataParser envelopedDataParser = new CMSEnvelopedDataParser(new
		// ByteArrayInputStream(encodedCms)); //ruslan

		CMSEnvelopedData enveloped = new CMSEnvelopedData(encodedCms);

		RecipientInformationStore recipients = enveloped.getRecipientInfos();
		java.util.Collection recc = recipients.getRecipients();
		RecipientInformation res = (RecipientInformation) recc.iterator().next();

		Security.addProvider(sHolder.getStore().getProvider());
		// System.out.println(sHolder.getStore().getProvider());
		Security.addProvider(new BouncyCastleProvider());
		byte[] content = null;

		Recipient recipient = new JceKeyTransEnvelopedRecipient(key);
		content = res.getContent(recipient);

//		if (sHolder.getStore().getProvider().getName().startsWith("SunPKCS11")
//				|| sHolder.getStore().getProvider().getName().startsWith("SunMSCAPI"))
//			content = res.getContent(key, sHolder.getStore().getProvider());
//		else
//			content = res.getContent(key, "BC");
		return content;
	}

	@SuppressWarnings("rawtypes")
	private void canonicalize(final org.dom4j.Element elem, final StringBuilder strb) {
		strb.append(elem.getName());

		if (elem.elements().isEmpty()) {
			boolean isEmpty = true;

			for (Iterator iter = elem.content().iterator(); iter.hasNext();) {
				Object o = iter.next();
				if (o instanceof AbstractText) {
					final String text = ((AbstractText) o).getText();
					if (!text.isEmpty()) {
						if (isEmpty) {
							isEmpty = false;
							strb.append("<=");
						}
						strb.append(text.replaceAll("\r\n", "\n"));
					}
				}
			}

			if (isEmpty)
				strb.append("<>");
			else
				strb.append("=>");
		} else {
			strb.append("<");
			for (Object o : elem.elements()) {
				canonicalize((org.dom4j.Element) o, strb);
			}
			strb.append(">");
		}
	}

	private byte[] sign(byte[] data, String algorithmID, PrivateKey privateKey, KeyStoreHolder storeHolder,
			int detached) throws OperatorCreationException, CertificateEncodingException, NoSuchAlgorithmException,
			NoSuchProviderException, CMSException, IOException {
		CMSTypedData typedData = new CMSProcessableByteArray(data);
		CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
		Security.addProvider(new BouncyCastleProvider());
		ContentSigner signer = null;
		// for jks default Sun provider
		if (storeHolder.getStore().getProvider().getName().equals("SUN"))
			signer = new JcaContentSignerBuilder((String) AlgorithmIDf.getOids().get(algorithmID)).setProvider("BC")
					.build(privateKey);
		// for pfx
		else if (storeHolder.getStore().getProvider().getName().equals("SunJSSE"))
			signer = new JcaContentSignerBuilder((String) AlgorithmIDf.getOids().get(algorithmID))
					.setProvider("SunRsaSign").build(privateKey);
		// mscapi + pkcs11 ..
		else
			signer = new JcaContentSignerBuilder((String) AlgorithmIDf.getOids().get(algorithmID))
					.setProvider(storeHolder.getStore().getProvider()).build(privateKey);
		JcaSignerInfoGeneratorBuilder builder = new JcaSignerInfoGeneratorBuilder(
				new JcaDigestCalculatorProviderBuilder().setProvider("BC").build());
		gen.addSignerInfoGenerator(builder.build(signer, storeHolder.getCertificate()));
		boolean isdetached = false;
		if (detached == 1) {
			isdetached = true;
		}

		CMSSignedData signed = gen.generate(typedData, isdetached); // "BC"
		signed = new CMSSignedData(signed.getEncoded());
		byte[] der = signed.getEncoded();
		return der;
	}

	/**/

	private static KeyPair generateECDSAKeyPair() {
		try {
			KeyPairGenerator kpGen = KeyPairGenerator.getInstance("ECDSA", "BC");
			ECGenParameterSpec ecSpec = new ECGenParameterSpec("prime192v1");
			kpGen.initialize(ecSpec, new SecureRandom());
			return kpGen.genKeyPair();
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchProviderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	/**/

	@SuppressWarnings("rawtypes")
	private KeyStoreHolder doFilter(List list, String serialNum, String issuerCN) {
		String serial = serialNum.replaceAll(" ", "");
		String issuer = issuerCN;
		for (int a = 0; a < list.size(); a++) {
			KeyStoreHolder model = (KeyStoreHolder) list.get(a);
			String iss = model.getCertificate().getIssuerDN().getName();
			BigInteger ser = new BigInteger(serial, 16);
			if (ser.equals(model.getCertificate().getSerialNumber()) && iss.equals(issuer)) {
				return model;
			}
		}
		return null;
	}

	private X509Certificate getCertFromString(String cert) throws CertificateException {
		String begin = "-----BEGIN CERTIFICATE-----";
		String end = "-----END CERTIFICATE-----";
		cert = cert.replaceAll("(\r\n|\n)", "");
		if (cert.startsWith(begin) && cert.endsWith(end)) {
			cert = cert.replace(begin, "");
			cert = cert.replace(end, "");
		}
		CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
		InputStream in = new ByteArrayInputStream(Base64Utils.base64Decode(cert));
		X509Certificate x509cert = (X509Certificate) certFactory.generateCertificate(in);
		return x509cert;
	}

	private static String getXmlPath(String line) {
		String newLine = "";
		String[] z = line.split("/");
		for (int i = 1; i < z.length - 1; i++) {
			newLine += z[i] + "/";
		}
		newLine = newLine.replaceAll("%20", " ");
		return newLine;
	}

	private static String getSerial(BigInteger SerialNumber) {
		String hexString = SerialNumber.toString(16);
		StringBuilder sb = new StringBuilder();

		while ((sb.length() + hexString.length()) < (2 * SerialNumber.toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		return sb.toString();
	}

	private static String getSerial(String serialNumber) {
		BigInteger bi = new BigInteger(serialNumber);
		String hexString = bi.toString(16);

		StringBuilder sb = new StringBuilder();

		while ((sb.length() + hexString.length()) < (2 * bi.toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		return sb.toString();
	}

	private String Xml2String(org.w3c.dom.Document document) throws TransformerException {
		TransformerFactory transformerFactory = TransformerFactory.newInstance();
		Transformer transformer = transformerFactory.newTransformer();
		DOMSource source = new DOMSource(document);
		StringWriter strWriter = new StringWriter();
		StreamResult result = new StreamResult(strWriter);
		transformer.transform(source, result);
		return strWriter.getBuffer().toString();
	}

	private org.w3c.dom.Document String2XmlDocument(String str)
			throws ParserConfigurationException, SAXException, IOException {

		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
		docBuilderFactory.setNamespaceAware(true);
		DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
		org.w3c.dom.Document document = docBuilder.parse(new InputSource(new StringReader(str)));

		return document;
	}

	private KeyStoreHolder getKeyStoreHolder(String cer_issuer, String cer_serial
	// , String hash_oid
			, String containerName, String containerPassword, String containerFilePath) throws KeyStoreException,
			GeneralSecurityException, IOException, NoSuchMethodException, ClassNotFoundException,
			InvocationTargetException, InstantiationException, IllegalAccessException, IllegalArgumentException {
		KeyStoreHolder storeHolder = null;// new KeyStoreHolder();
		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromWindows(), cer_serial, cer_issuer);
		} else if (containerName.equals(PKCS11_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS11(containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS12(containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(JKS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromJKS(containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else {
			try {
				throw new Throwable("Container has not been declared");
			} catch (Throwable e) {
			}
		}
		return storeHolder;
	}

	public static void installCertificateToStore(String containerName, String containerFilePath,
			String containerPassword, X509Certificate certificate) {
		try {
			if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
				// X509Certificate cert = getCertFromString(certificate);
				KeyStore ks = KeyStore.getInstance("Windows-My");
				// KeyStore ks = KeyStore.getInstance(JKS_FILE);
				ks.load(null, null);
				ks.setCertificateEntry(certificate.getSerialNumber().toString(16), certificate);
			} else if (containerName.equals(JKS_CONTAINER_NAME)) {
				FileInputStream is = new FileInputStream(containerFilePath);
				KeyStore keystore = KeyStore.getInstance("JKS");
				keystore.load(is, containerPassword.toCharArray());

				String alias = certificate.getSerialNumber().toString(16);
				char[] password = containerPassword.toCharArray();

				Certificate certs = certificate;
				File keystoreFile = new File(containerFilePath);
				// Load the keystore contents
				FileInputStream in = new FileInputStream(keystoreFile);
				keystore.load(in, password);
				in.close();

				// Add the certificate
				keystore.setCertificateEntry(alias, certs);

				// Save the new keystore contents
				FileOutputStream out = new FileOutputStream(keystoreFile);
				keystore.store(out, password);
				out.close();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String CorrectIssuerDN(String issuerDN) {
        StringBuilder sb = new StringBuilder();
        String[] lst = issuerDN.split(",");

        for (int i = 0; i < lst.length; i++)
        {
        	String s = lst[i];
            s = s.trim();
            s = s.replace("cn=", "CN=");
            s = s.replace("cN=", "CN=");
            s = s.replace("Cn=", "CN=");
            
            s = s.replace("ou=", "OU=");
            s = s.replace("oU=", "OU=");
            s = s.replace("Ou=", "OU=");
            
            s = s.replace("o=", "O=");
            s = s.replace("c=", "C=");
            
            if (i == lst.length-1)
            	sb.append(s);
            else
            	sb.append(s + ", ");
            
        }
        String res = sb.toString().trim().trim();
        return res;
    }

	// ****************************************************************
	// PRIVATE METHODS END *
	// ****************************************************************

	//// lazimsiz metodlar
	@SuppressWarnings({ "unused", "rawtypes" })
	private String decrypteData(final String inputdata) throws KeyStoreException, GeneralSecurityException, IOException,
			SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException, InvocationTargetException, OperatorCreationException,
			CMSException, ParserConfigurationException, SAXException {
		Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
		String cer_issuer = (String) xmlMap.get("cer_issuer");
		String cer_serial = (String) xmlMap.get("cer_serial");
		String hash_oid = (String) xmlMap.get("hash_oid");
		String containerName = (String) xmlMap.get("containerName");
		String containerPassword = (String) xmlMap.get("containerPassword");
		String containerFilePath = (String) xmlMap.get("containerFilePath");
		KeyStoreHolder storeHolder = new KeyStoreHolder();
		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromWindows(), cer_serial, cer_issuer);
		} else if (containerName.equals(PKCS11_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS11(containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS12(containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(JKS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromJKS(containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else {
			try {
				throw new Throwable("Container has not been declared");
			} catch (Throwable e) {
			}
		}
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null && !containerPassword.isEmpty() && !containerPassword.equals(""))
				passw = containerPassword.toCharArray();

			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(storeHolder.getAlias(), passw);
			return Base64Utils.base64Encode(decrypt(inputdata, pk, storeHolder));

		}

		return null;
	}

	private X509Certificate getCertFromCerFile(final String file_path) throws CertificateException, IOException {

		CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
		InputStream in = new ByteArrayInputStream(XmlReader.fileToByteArray(file_path));
		X509Certificate x509cert = (X509Certificate) certFactory.generateCertificate(in);
		return x509cert;
	}

	public void InstallCert() throws Exception {
		X509Certificate cert = getCertFromCerFile("D:\\\\memmed.cer");
		KeyStore ks = KeyStore.getInstance("Windows-My");
		// KeyStore ks = KeyStore.getInstance(JKS_FILE);
		ks.load(null, null);
		ks.setCertificateEntry(cert.getSerialNumber().toString(), cert);

	}

	/*
	 * public X509Certificate GetCertificateFromStore(String containerName, String
	 * issuer, String serial) throws Exception{ KeyStore ks =
	 * KeyStore.getInstance("Windows-My"); ks.load(null, null); X509Certificate cer
	 * = (X509Certificate)ks.getCertificate(serial); return cer; }
	 */

	@SuppressWarnings({ "unused", "rawtypes" })
	private Provider getProvider(final String name)
			throws ParserConfigurationException, SAXException, IOException, KeyStoreException {
		String containerName = "";
		KeyStore ks = null;
		if (name == null) {
			Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
			containerName = (String) xmlMap.get("containerName");
		} else {
			containerName = name;
		}

		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			ks = KeyStore.getInstance("Windows-My");
		}

		else if (containerName.equals(PKCS11_CONTAINER_NAME)) {

			ks = KeyStore.getInstance("PKCS11");
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			ks = KeyStore.getInstance("PKCS12");
		}

		return ks.getProvider();

	}

	@SuppressWarnings("unused")
	private String getSerial(final X509Certificate cert) {
		String hexString = cert.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();

		while ((sb.length() + hexString.length()) < (2 * cert.getSerialNumber().toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		return sb.toString();
	}

}
