package com.crypto.utils.helpers;

import java.util.HashMap;
import java.util.Map;

import javax.xml.crypto.dsig.DigestMethod;
import javax.xml.crypto.dsig.SignatureMethod;

import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.X509ObjectIdentifiers;
import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;

@SuppressWarnings("unchecked")
public class AlgorithmIDf {
	@SuppressWarnings("rawtypes")
	private static final Map oids = new HashMap();
	@SuppressWarnings("rawtypes")
	private static final Map encryptionAlgs = new HashMap();
	@SuppressWarnings("rawtypes")
	private static final Map digestAlgs = new HashMap();
	static {
		oids.put(PKCSObjectIdentifiers.sha1WithRSAEncryption.toString(), "SHA1withRSA");
		oids.put(PKCSObjectIdentifiers.sha224WithRSAEncryption.toString(), "SHA224WITHRSA");
		oids.put(PKCSObjectIdentifiers.sha256WithRSAEncryption.toString(), "SHA256WITHRSA");
		oids.put(PKCSObjectIdentifiers.sha384WithRSAEncryption.toString(), "SHA384WITHRSA");
		oids.put(PKCSObjectIdentifiers.sha512WithRSAEncryption.toString(), "SHA512WITHRSA");

		oids.put(X9ObjectIdentifiers.ecdsa_with_SHA1.toString(), "SHA1WITHECDSA");
		oids.put(X9ObjectIdentifiers.ecdsa_with_SHA224.toString(), "SHA224WITHECDSA");
		oids.put(X9ObjectIdentifiers.ecdsa_with_SHA256.toString(), "SHA256WITHECDSA");
		oids.put(X9ObjectIdentifiers.ecdsa_with_SHA384.toString(), "SHA384WITHECDSA");
		oids.put(X9ObjectIdentifiers.ecdsa_with_SHA512.toString(), "SHA512WITHECDSA");
		oids.put(OIWObjectIdentifiers.dsaWithSHA1.toString(), "SHA1WITHDSA");
		oids.put(NISTObjectIdentifiers.dsa_with_sha224.toString(), "SHA224WITHDSA");
		oids.put(NISTObjectIdentifiers.dsa_with_sha256.toString(), "SHA256WITHDSA");
		encryptionAlgs.put(X9ObjectIdentifiers.id_dsa, "DSA");
		encryptionAlgs.put(PKCSObjectIdentifiers.rsaEncryption, "RSA");

		encryptionAlgs.put(TeleTrusTObjectIdentifiers.teleTrusTRSAsignatureAlgorithm, "RSA");
		encryptionAlgs.put(X509ObjectIdentifiers.id_ea_rsa, "RSA");
		encryptionAlgs.put(PKCSObjectIdentifiers.id_RSASSA_PSS, "RSAandMGF1");
		encryptionAlgs.put(CryptoProObjectIdentifiers.gostR3410_94, "GOST3410");
		encryptionAlgs.put(CryptoProObjectIdentifiers.gostR3410_2001, "ECGOST3410");
		encryptionAlgs.put(new ASN1ObjectIdentifier("1.3.6.1.4.1.5849.1.6.2"), "ECGOST3410");
		encryptionAlgs.put(new ASN1ObjectIdentifier("1.3.6.1.4.1.5849.1.1.5"), "GOST3410");
		encryptionAlgs.put(CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_2001, "ECGOST3410");
		encryptionAlgs.put(CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_94, "GOST3410");

		digestAlgs.put(PKCSObjectIdentifiers.md2, "MD2");
		digestAlgs.put(PKCSObjectIdentifiers.md4, "MD4");
		digestAlgs.put(PKCSObjectIdentifiers.md5, "MD5");
		digestAlgs.put(OIWObjectIdentifiers.idSHA1, "SHA1");
		digestAlgs.put(NISTObjectIdentifiers.id_sha224, "SHA224");
		digestAlgs.put(NISTObjectIdentifiers.id_sha256, "SHA256");
		digestAlgs.put(NISTObjectIdentifiers.id_sha384, "SHA384");
		digestAlgs.put(NISTObjectIdentifiers.id_sha512, "SHA512");
		digestAlgs.put(TeleTrusTObjectIdentifiers.ripemd128, "RIPEMD128");
		digestAlgs.put(TeleTrusTObjectIdentifiers.ripemd160, "RIPEMD160");
		digestAlgs.put(TeleTrusTObjectIdentifiers.ripemd256, "RIPEMD256");
		digestAlgs.put(CryptoProObjectIdentifiers.gostR3411, "GOST3411");
		digestAlgs.put(new ASN1ObjectIdentifier("1.3.6.1.4.1.5849.1.2.1"), "GOST3411");
	}

	public static String getSignatureAlgorithmName(AlgorithmIdentifier digest, AlgorithmIdentifier encrypt) {
		return digestAlgs.get(digest.getAlgorithm()) + "with" + encryptionAlgs.get(encrypt.getAlgorithm());
	}

	@SuppressWarnings("rawtypes")
	public static Map getOids() {
		return oids;
	}

	public static String getDigestMethodNameByHashOID(String hashOID) {
		String algName = "";
		switch (hashOID) {
		case "1.2.840.113549.1.1.5":
			algName = DigestMethod.SHA1;
			break;
		case "1.2.840.113549.1.1.11":
			algName = DigestMethod.SHA256;
			break;
		case "1.2.840.113549.1.1.13":
			algName = DigestMethod.SHA512;
			break;
		default:
			algName = "";
		}
		return algName;
	}

	public static String getSignatureMethodNameByHashOID(String hashOID) {
		String algName = "";
		switch (hashOID) {
		case "1.2.840.113549.1.1.5":
			algName = SignatureMethod.RSA_SHA1;
			break;
		case "1.2.840.113549.1.1.11":
			algName = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256";
			break;
		case "1.2.840.113549.1.1.13":
			algName = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512";
			break;
		default:
			algName = "";
		}
		return algName;
	}

}