package com.crypto.utils.helpers;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.KeyStore;
import java.security.cert.CRLException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Hashtable;

import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.x500.AttributeTypeAndValue;
import org.bouncycastle.asn1.x500.RDN;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x500.style.BCStyle;
import org.bouncycastle.asn1.x500.style.IETFUtils;

//import com.crypto.utils.AzTrustCryptoLib;

public class CerHelper {
	
	public static X509Certificate readCertFromFile(String filePath)
			throws FileNotFoundException, IOException, CertificateException, CRLException {
		InputStream inStream = new ByteArrayInputStream(XmlReader.fileToByteArray(filePath));
		CertificateFactory cf = CertificateFactory.getInstance("X.509");
		X509Certificate cert = (X509Certificate) cf.generateCertificate(inStream);
		return cert;
	}

	public static X509Certificate downloadCerFromWeb(String crlURL)
			throws MalformedURLException, IOException, CertificateException, CRLException {
		URL url = new URL(crlURL);
		InputStream cerStream = url.openStream();
		try {
			CertificateFactory cf = CertificateFactory.getInstance("X.509");
			X509Certificate cer = (X509Certificate) cf.generateCertificate(cerStream);
			return cer;
		} finally {
			cerStream.close();
		}
	}
	
	public static X509Certificate getCertificateFromStore(String containerName, String issuer, String serialNumber) {
		try {
			KeyStore ks = KeyStore.getInstance("Windows-My");
			ks.load(null, null);
			
			BigInteger b = new BigInteger(serialNumber, 16);
			String serial = b.toString(16);
			X509Certificate cer = (X509Certificate)ks.getCertificate(serial.toString().trim());
			//ks.
			return cer;
		}
		catch (Exception e) {
			return null;
		}
	}
	
	//lib-de silmek
	public static X509Certificate getCertFromString(String cert) throws CertificateException {
		String begin = "-----BEGIN CERTIFICATE-----";
		String end = "-----END CERTIFICATE-----";
		cert = cert.replaceAll("(\r\n|\n)", "");
		if (cert.startsWith(begin) && cert.endsWith(end)) {
			cert = cert.replace(begin, "");
			cert = cert.replace(end, "");
		}
		CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
		InputStream in = new ByteArrayInputStream(Base64Utils.base64Decode(cert));
		X509Certificate x509cert = (X509Certificate) certFactory.generateCertificate(in);
		return x509cert;
	}
	
	public static String correctIssuerDN(String issuerDN) {
        StringBuilder sb = new StringBuilder();
        String[] lst = issuerDN.split(",");

        for (int i = 0; i < lst.length; i++)
        {
        	String s = lst[i];
            s = s.trim();
            s = s.replace("cn=", "CN=");
            s = s.replace("ou=", "OU=");
            s = s.replace("o=", "O=");
            s = s.replace("c=", "C=");
            
            if (i == lst.length-1)
            	sb.append(s);
            else
            	sb.append(s + ", ");
            
        }
        String res = sb.toString().trim().trim();
        return res;
    }	

	private static final Hashtable<ASN1ObjectIdentifier, String> DefaultSymbols = 
			new Hashtable<>();
	
	static
    {
        DefaultSymbols.put(BCStyle.C, "c");
        DefaultSymbols.put(BCStyle.O, "o");
        DefaultSymbols.put(BCStyle.T, "title");
        DefaultSymbols.put(BCStyle.OU, "ou");
        DefaultSymbols.put(BCStyle.CN, "cn");
        DefaultSymbols.put(BCStyle.L, "l");
        DefaultSymbols.put(BCStyle.ST, "st");
        DefaultSymbols.put(BCStyle.SN, "serialNumber");
        DefaultSymbols.put(BCStyle.EmailAddress, "e");
        DefaultSymbols.put(BCStyle.DC, "dc");
        DefaultSymbols.put(BCStyle.UID, "uid");
        DefaultSymbols.put(BCStyle.STREET, "street");
        DefaultSymbols.put(BCStyle.SURNAME, "sn");
        DefaultSymbols.put(BCStyle.GIVENNAME, "givenName");
        DefaultSymbols.put(BCStyle.INITIALS, "initials");
        DefaultSymbols.put(BCStyle.GENERATION, "generation");
        DefaultSymbols.put(BCStyle.UnstructuredAddress, "unstructuredAddress");
        DefaultSymbols.put(BCStyle.UnstructuredName, "unstructuredName");
        DefaultSymbols.put(BCStyle.UNIQUE_IDENTIFIER, "uniqueIdentifier");
        DefaultSymbols.put(BCStyle.DN_QUALIFIER, "dn");
        DefaultSymbols.put(BCStyle.PSEUDONYM, "pseudonym");
        DefaultSymbols.put(BCStyle.POSTAL_ADDRESS, "postalAddress");
        DefaultSymbols.put(BCStyle.NAME_AT_BIRTH, "nameAtBirth");
        DefaultSymbols.put(BCStyle.COUNTRY_OF_CITIZENSHIP, "countryOfCitizenship");
        DefaultSymbols.put(BCStyle.COUNTRY_OF_RESIDENCE, "countryOfResidence");
        DefaultSymbols.put(BCStyle.GENDER, "gender");
        DefaultSymbols.put(BCStyle.PLACE_OF_BIRTH, "placeOfBirth");
        DefaultSymbols.put(BCStyle.DATE_OF_BIRTH, "dateOfBirth");
        DefaultSymbols.put(BCStyle.POSTAL_CODE, "postalCode");
        DefaultSymbols.put(BCStyle.BUSINESS_CATEGORY, "businessCategory");
        DefaultSymbols.put(BCStyle.TELEPHONE_NUMBER, "telephoneNumber");
        DefaultSymbols.put(BCStyle.NAME, "name");
        
    }
	
    public static void appendTypeAndValue(
    		StringBuilder buf,
            AttributeTypeAndValue typeAndValue,
            Hashtable<ASN1ObjectIdentifier, String> oidSymbols)
    {
        String  sym = oidSymbols.get(typeAndValue.getType());

        if (sym != null)
        {
            buf.append(sym);
        }
        else
        {
            buf.append(typeAndValue.getType().getId());
        }

        buf.append('=');

        buf.append(IETFUtils.valueToString(typeAndValue.getValue()));
    }
    
    public static String toStringReversed(X500Name name)
    {
    	final StringBuilder buf = new StringBuilder();
		boolean first = true;
		final RDN[] rdns = name.getRDNs(); 
		for (int i = rdns.length-1; i >= 0; i--) {
			if (first)
            {
                first = false;
            }
            else
            {
                buf.append(',');
            }
			if (rdns[i].isMultiValued())
            {
                AttributeTypeAndValue[] atv = rdns[i].getTypesAndValues();
                boolean firstAtv = true;

                for (int j = 0; j != atv.length; j++)
                {
                    if (firstAtv)
                    {
                        firstAtv = false;
                    }
                    else
                    {
                        buf.append('+');
                    }
                    
                    appendTypeAndValue(buf, atv[j], DefaultSymbols);
                }
            }
            else
            {
            	appendTypeAndValue(buf, rdns[i].getFirst(), DefaultSymbols);
            }
		}
		return buf.toString();
    }

}
