package com.crypto.utils.helpers;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

public class LdapReader {
 
	public static  X509Certificate getCertificate(
		  String ldapServer
		 ,String ldapPort
		 ,String ldapUser
		 ,String ldapPassword 
		 ,String ldapCertSearchDN
		 ,String serialNumber
		 ,String Issuer) 
			 throws CertificateException, NamingException {
	 
	 X509Certificate cert=null;
	 LdapContext ctx=  login(ldapServer,ldapPort,ldapUser,ldapPassword);       
	 byte[] bytes= search(serialNumber,Issuer,ctx,ldapCertSearchDN);
	 CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
	 InputStream in = new ByteArrayInputStream(bytes);
	 cert = (X509Certificate)certFactory.generateCertificate(in);
	
	 

	 /*
	 X509Certificate cert=null;
	 try {
			cert = CerHelper.readCertFromFile("D:\\XadesDocs\\Cert\\aaa.cer");
	 } 
	 catch (Exception e) {
		// TODO Auto-generated catch block
		 e.printStackTrace();
	 } 
	 */
     return cert;
  }
 
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private static LdapContext login(
		  String ldapServer
		, String ldapPort
		, String ldapUser
		, String ldapPassword)
			throws NamingException {
		Hashtable authEnv = new Hashtable();
		String ldapURL = "ldap://" + ldapServer + ":" + ldapPort;
		authEnv.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
		authEnv.put(Context.PROVIDER_URL, ldapURL);
		authEnv.put(Context.SECURITY_AUTHENTICATION, "simple");
		authEnv.put(Context.SECURITY_PRINCIPAL, ldapUser);
		authEnv.put(Context.SECURITY_CREDENTIALS, ldapPassword);

		return new InitialLdapContext(authEnv, null);
	}
  
	@SuppressWarnings("rawtypes")
	private static byte[] search(String serialNumber, String Issuer, LdapContext ctx, String dir)
			throws NamingException, CertificateException {
		SearchControls sc = new SearchControls();
		String[] attributeFilter = { "attributeCertificateAttribute", "userCertificate" };
		sc.setReturningAttributes(attributeFilter);
		sc.setSearchScope(SearchControls.SUBTREE_SCOPE);

		String filter = "(&(attributeCertificateAttribute=Issuer=" + Issuer
				+ ")(attributeCertificateAttribute=serialNumber=" + serialNumber + "))";

		NamingEnumeration results = ctx.search(dir, filter, sc);

		while (results.hasMore()) {
			SearchResult sr = (SearchResult) results.next();
			Attributes attrs = sr.getAttributes();
			Attribute attrCer = attrs.get("userCertificate");
			if (attrCer != null) {
				return (byte[]) attrCer.get(0);
			}
		}
		ctx.close();
		return null;
	}

}
 

